#pragma once

#include <list>

#include "page_mapper.h"
#include "pages_conserving.h"
#include "pipe_accessor.h"

/* Pipe paging support, maintaining the sequence of active regions or sections
   in a pipe. */

class PipePaging
{
protected:
    Memory *_memory;
    PagesConserving *_pages;

    /* Regions acting as files with their own accessors. */

    std::list<PageMapper *> _regions;
    std::list<PipeAccessor> _accessors;

    /* Pipe section/region size. */

    offset_t _size;

    /* Endpoint status. */

    unsigned int _endpoints = 2;

public:
    explicit PipePaging(Memory *memory, offset_t size);

    explicit PipePaging(offset_t size);

    virtual void detach();

    virtual PagesConserving *pages()
    { return _pages; }

    virtual offset_t region_size()
    { return _size; }

    /* Region management. */

    virtual void add_region(PageMapper *mapper);

    virtual PageMapper *first_region();

    virtual PageMapper *next_region();

    virtual PipeAccessor *accessor();
};

// vim: tabstop=4 expandtab shiftwidth=4
